(herald "Four Party Needham-Schroeder-Lowe Protocol")

(comment "CPSA 2.3.3")
(comment "All input read from nsl4cm1.lsp")

(defprotocol nsl4cm basic
  (defrole init
    (vars (a b c d name) (na nb nc nd text))
    (trace (send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b)))))
  (defrole resp1
    (vars (a b c d name) (na nb nc nd text))
    (trace (recv (enc a c d na (pubk b)))
      (send (enc a b d na nb (pubk c))) (recv (enc nb nc nd (pubk b)))
      (send (enc nc nd (pubk c)))))
  (defrole resp2
    (vars (a b c d name) (na nb nc nd text))
    (trace (recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))) (recv (enc nc nd (pubk c)))
      (send (enc nd (pubk d)))))
  (defrole resp3
    (vars (a b c d name) (na nb nc nd text))
    (trace (recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a)))
      (recv (enc nd (pubk d))))))

(defskeleton nsl4cm
  (vars (na nb nc nd text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b)))))
  (label 0)
  (unrealized (0 1))
  (origs (na (0 0)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1)
  (parent 0)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 2)
  (parent 1)
  (unrealized (0 1) (2 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 3 1 resp1 2) na (2 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 3)
  (parent 2)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (2 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 4)
  (parent 2)
  (seen 2)
  (unrealized (0 1) (2 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 5)
  (parent 3)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 6)
  (parent 3)
  (seen 3)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 7)
  (parent 3)
  (unrealized (0 1) (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 8)
  (parent 5)
  (seen 3)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 9)
  (parent 5)
  (seen 5)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 10)
  (parent 5)
  (seen 12)
  (unrealized (0 1) (4 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 4 1 resp1 2) na (3 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 11)
  (parent 7)
  (unrealized (0 1) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 12)
  (parent 7)
  (seen 15)
  (unrealized (0 1) (3 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 13)
  (parent 10)
  (seen 15)
  (unrealized (0 1) (4 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 14)
  (parent 10)
  (seen 19)
  (unrealized (0 1) (4 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 15)
  (parent 11)
  (unrealized (0 1) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 4 2 resp2 2) na (3 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 16)
  (parent 11)
  (seen 20)
  (unrealized (0 1))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 17)
  (parent 11)
  (unrealized (0 1) (3 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 18)
  (parent 12)
  (unrealized (0 1) (3 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 19)
  (parent 13)
  (unrealized (0 1) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 2 resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 20)
  (parent 13)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 21)
  (parent 13)
  (seen 31)
  (unrealized (0 1) (4 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (4 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 22)
  (parent 14)
  (seen 35)
  (unrealized (0 1) (4 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 23)
  (parent 14)
  (seen 34)
  (unrealized (0 1) (4 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 24)
  (parent 15)
  (seen 32)
  (unrealized (0 1) (3 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 2 resp2 2) na (3 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 25)
  (parent 15)
  (seen 37)
  (unrealized (0 1))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 26)
  (parent 15)
  (seen 31)
  (unrealized (0 1) (3 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a)))))
  (label 27)
  (parent 16)
  (unrealized)
  (shape)
  (maps ((0) ((a a) (b b) (c c) (d d) (na na) (nb nb) (nc nc) (nd nd))))
  (origs (na (0 0))))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 28)
  (parent 16)
  (unrealized (0 1) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 29)
  (parent 16)
  (unrealized (0 1) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 30)
  (parent 17)
  (seen 56)
  (unrealized (0 1) (3 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 31)
  (parent 17)
  (unrealized (0 1) (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 32)
  (parent 18)
  (unrealized (0 1) (3 0) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 33)
  (parent 18)
  (seen 7)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 34)
  (parent 18)
  (seen 70)
  (unrealized (0 1) (3 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 35)
  (parent 19)
  (unrealized (0 1) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 36)
  (parent 19)
  (seen 47 51)
  (unrealized (0 1) (4 0) (6 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 37)
  (parent 19)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 38)
  (parent 19)
  (seen 44 57)
  (unrealized (0 1) (4 0) (6 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a)))))
  (label 39)
  (parent 20)
  (seen 27)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 40)
  (parent 20)
  (unrealized (0 1) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 41)
  (parent 20)
  (seen 59)
  (unrealized (0 1) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 42)
  (parent 20)
  (seen 61)
  (unrealized (0 1) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 43)
  (parent 21)
  (seen 56 86)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 44)
  (parent 21)
  (unrealized (0 1) (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 45)
  (parent 22)
  (unrealized (0 1) (4 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 46)
  (parent 22)
  (unrealized (0 1) (4 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 47)
  (parent 23)
  (seen 70)
  (unrealized (0 1) (4 0) (6 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 48)
  (parent 23)
  (seen 10)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 50)
  (parent 24)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 51)
  (parent 24)
  (seen 70)
  (unrealized (0 1) (3 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 52)
  (parent 25)
  (seen 27)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 53)
  (parent 25)
  (unrealized (0 1) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 54)
  (parent 25)
  (unrealized (0 1) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 55)
  (parent 25)
  (unrealized (0 1) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 56)
  (parent 26)
  (unrealized (0 1) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 57)
  (parent 26)
  (seen 99)
  (unrealized (0 1) (3 0) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 58)
  (parent 28)
  (seen 91)
  (unrealized (0 1))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 59)
  (parent 28)
  (unrealized (0 1) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 60)
  (parent 29)
  (seen 16)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 61)
  (parent 29)
  (unrealized (0 1) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 62)
  (parent 30)
  (seen 95)
  (unrealized (0 1))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 63)
  (parent 30)
  (seen 30)
  (unrealized (0 1) (3 0) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 64)
  (parent 30)
  (unrealized (0 1) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 65)
  (parent 31)
  (seen 115 135)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 66)
  (parent 31)
  (seen 136)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 67)
  (parent 31)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 68)
  (parent 32)
  (unrealized (0 1) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 69)
  (parent 32)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 70)
  (parent 32)
  (unrealized (0 1) (3 0) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 71)
  (parent 32)
  (seen 152 175)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 72)
  (parent 32)
  (unrealized (0 1) (3 0) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 73)
  (parent 34)
  (seen 120 152)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 75)
  (parent 35)
  (seen 102)
  (unrealized (0 1) (4 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 76)
  (parent 35)
  (seen 106)
  (unrealized (0 1) (4 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (4 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 77)
  (parent 35)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 78)
  (parent 35)
  (unrealized (0 1) (4 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 79)
  (parent 36)
  (seen 116)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 81)
  (parent 37)
  (seen 52)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 82)
  (parent 37)
  (seen 124)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 83)
  (parent 37)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 84)
  (parent 37)
  (seen 127)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 85)
  (parent 37)
  (seen 130)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 86)
  (parent 38)
  (seen 198)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 88)
  (parent 40)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 89)
  (parent 40)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 90)
  (parent 40)
  (seen 40)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 91)
  (parent 41)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 92)
  (parent 41)
  (unrealized (0 1) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 93)
  (parent 42)
  (seen 20)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 94)
  (parent 42)
  (unrealized (0 1) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-1 nc-0)) na (4 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 95)
  (parent 43)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 96)
  (parent 43)
  (seen 43)
  (unrealized (0 1) (4 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 97)
  (parent 43)
  (seen 151)
  (unrealized (0 1) (4 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 98)
  (parent 44)
  (seen 135 202)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 99)
  (parent 44)
  (unrealized (0 1) (4 0) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 100)
  (parent 44)
  (seen 136)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 102)
  (parent 45)
  (unrealized (0 1) (4 0) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 103)
  (parent 45)
  (seen 269)
  (unrealized (0 1) (4 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 104)
  (parent 45)
  (seen 187)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 106)
  (parent 46)
  (unrealized (0 1) (4 0) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 107)
  (parent 46)
  (seen 189)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 108)
  (parent 46)
  (seen 285)
  (unrealized (0 1) (4 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 110)
  (parent 47)
  (unrealized (0 1) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 111)
  (parent 47)
  (seen 167)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 113)
  (parent 47)
  (seen 175 254)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 115)
  (parent 50)
  (seen 202)
  (unrealized (0 1))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 116)
  (parent 50)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 117)
  (parent 50)
  (seen 50)
  (unrealized (0 1) (3 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 118)
  (parent 50)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 119)
  (parent 50)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 120)
  (parent 51)
  (seen 152)
  (unrealized (0 1) (3 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 122)
  (parent 53)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 123)
  (parent 53)
  (seen 208)
  (unrealized (0 1))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 124)
  (parent 53)
  (seen 334)
  (unrealized (0 1) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 125)
  (parent 54)
  (seen 214)
  (unrealized (0 1))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 126)
  (parent 54)
  (unrealized (0 1) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 127)
  (parent 54)
  (seen 352)
  (unrealized (0 1) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 128)
  (parent 55)
  (unrealized (0 1) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 129)
  (parent 55)
  (seen 357)
  (unrealized (0 1) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 130)
  (parent 55)
  (seen 358 361)
  (unrealized (0 1) (5 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 131)
  (parent 56)
  (seen 220)
  (unrealized (0 1))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 132)
  (parent 56)
  (seen 320)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 133)
  (parent 56)
  (seen 56)
  (unrealized (0 1) (3 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 134)
  (parent 56)
  (seen 151)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 135)
  (parent 57)
  (seen 315)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 136)
  (parent 57)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 138)
  (parent 58)
  (seen 27)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 139)
  (parent 58)
  (seen 58)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 140)
  (parent 58)
  (unrealized (0 1) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 141)
  (parent 58)
  (unrealized (0 1) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 142)
  (parent 59)
  (seen 240)
  (unrealized (0 1))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 143)
  (parent 59)
  (seen 242)
  (unrealized (0 1))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 144)
  (parent 59)
  (unrealized (0 1) (4 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 145)
  (parent 61)
  (seen 359)
  (unrealized (0 1) (4 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nd-1 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 146)
  (parent 61)
  (seen 29)
  (unrealized (0 1) (4 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nd nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d)))))
  (label 147)
  (parent 62)
  (seen 27)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 148)
  (parent 62)
  (seen 62)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 149)
  (parent 62)
  (seen 11)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 150)
  (parent 64)
  (unrealized (0 1) (3 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 151)
  (parent 64)
  (seen 401)
  (unrealized (0 1) (3 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 152)
  (parent 65)
  (seen 254)
  (unrealized (0 1))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 153)
  (parent 65)
  (seen 65)
  (unrealized (0 1) (3 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 154)
  (parent 65)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 155)
  (parent 66)
  (seen 262)
  (unrealized (0 1))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 156)
  (parent 66)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 157)
  (parent 66)
  (seen 265)
  (unrealized (0 1))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 158)
  (parent 66)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 159)
  (parent 66)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 160)
  (parent 67)
  (unrealized (0 1) (3 0) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 161)
  (parent 67)
  (seen 65)
  (unrealized (0 1) (3 0) (4 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 163)
  (parent 68)
  (seen 298)
  (unrealized (0 1))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 164)
  (parent 68)
  (seen 68)
  (unrealized (0 1) (3 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 165)
  (parent 68)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 166)
  (parent 69)
  (seen 302)
  (unrealized (0 1))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 167)
  (parent 69)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 168)
  (parent 69)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 169)
  (parent 69)
  (seen 305)
  (unrealized (0 1))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 170)
  (parent 69)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 171)
  (parent 69)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 172)
  (parent 70)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 173)
  (parent 70)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 175)
  (parent 70)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 177)
  (parent 71)
  (seen 308)
  (unrealized (0 1))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 178)
  (parent 71)
  (seen 71)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 179)
  (parent 71)
  (unrealized (0 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 180)
  (parent 72)
  (seen 68)
  (unrealized (0 1) (3 0) (5 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 181)
  (parent 72)
  (unrealized (0 1) (3 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 183)
  (parent 73)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 186)
  (parent 75)
  (seen 269)
  (unrealized (0 1) (4 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 187)
  (parent 75)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 189)
  (parent 76)
  (seen 292)
  (unrealized (0 1) (4 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 190)
  (parent 76)
  (seen 285)
  (unrealized (0 1) (4 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-0 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 192)
  (parent 77)
  (seen 52)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nb-3 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 193)
  (parent 77)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 194)
  (parent 77)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 195)
  (parent 77)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 196)
  (parent 77)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 197)
  (parent 77)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 198)
  (parent 78)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 199)
  (parent 78)
  (seen 99)
  (unrealized (0 1) (4 0) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 200)
  (parent 78)
  (seen 99)
  (unrealized (0 1) (4 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1) (nc-1 nc-0)) na (4 0)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d)))))
  (label 202)
  (parent 79)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 207)
  (parent 82)
  (seen 334)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 208)
  (parent 82)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 210)
  (parent 83)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 211)
  (parent 83)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 212)
  (parent 83)
  (seen 574)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 214)
  (parent 84)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 215)
  (parent 84)
  (seen 352)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 217)
  (parent 85)
  (seen 358)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 218)
  (parent 85)
  (seen 361 599)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nb-2 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-1 nc-0)) na (4 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 220)
  (parent 86)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 224)
  (parent 88)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 225)
  (parent 88)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 226)
  (parent 88)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 227)
  (parent 88)
  (seen 623)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 228)
  (parent 88)
  (unrealized (0 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 229)
  (parent 89)
  (seen 39)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 230)
  (parent 89)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 231)
  (parent 89)
  (seen 89)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 232)
  (parent 89)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 233)
  (parent 89)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 234)
  (parent 89)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 235)
  (parent 91)
  (seen 138)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 236)
  (parent 91)
  (seen 639)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 237)
  (parent 91)
  (seen 91)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 238)
  (parent 91)
  (seen 384)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 239)
  (parent 91)
  (seen 386)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 240)
  (parent 92)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 241)
  (parent 92)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 242)
  (parent 92)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 244)
  (parent 94)
  (unrealized (0 1) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 245)
  (parent 94)
  (seen 601 670)
  (unrealized (0 1) (5 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nb-0 nd nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d)))))
  (label 247)
  (parent 95)
  (seen 39)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 248)
  (parent 95)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 249)
  (parent 95)
  (seen 95)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 250)
  (parent 95)
  (seen 13)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 251)
  (parent 97)
  (seen 401)
  (unrealized (0 1) (4 0) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (4 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 254)
  (parent 98)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 257)
  (parent 99)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 258)
  (parent 99)
  (seen 688)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 259)
  (parent 99)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-1 nc-0)) na (4 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 262)
  (parent 100)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 265)
  (parent 100)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 268)
  (parent 102)
  (unrealized (0 1) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 269)
  (parent 102)
  (unrealized (0 1) (4 0) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 270)
  (parent 102)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 272)
  (parent 102)
  (seen 279 724)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 274)
  (parent 103)
  (seen 504 688)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d)))))
  (label 277)
  (parent 104)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c)))))
  (label 279)
  (parent 104)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 283)
  (parent 106)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 284)
  (parent 106)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 285)
  (parent 106)
  (unrealized (0 1) (4 0) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 287)
  (parent 106)
  (seen 292 768)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 289)
  (parent 107)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-1 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 292)
  (parent 107)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 295)
  (parent 108)
  (seen 184 520 688)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 296)
  (parent 108)
  (seen 521 688)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-1 nc-0)) na (4 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 298)
  (parent 110)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 299)
  (parent 110)
  (unrealized (0 1) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1) (nc-1 nc-0)) na (4 0)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d)))))
  (label 302)
  (parent 111)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (4 0)
    (enc a b c na nb-2 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 305)
  (parent 111)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 308)
  (parent 113)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nd nb nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d)))))
  (label 312)
  (parent 115)
  (seen 406)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 313)
  (parent 115)
  (seen 115)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 314)
  (parent 115)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0) (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 315)
  (parent 116)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 320)
  (parent 118)
  (unrealized (0 1) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 321)
  (parent 118)
  (seen 65 161 417)
  (unrealized (0 1) (3 0) (6 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 323)
  (parent 119)
  (unrealized (0 1) (3 0) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 324)
  (parent 119)
  (unrealized (0 1) (3 0) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 326)
  (parent 120)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 329)
  (parent 120)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 332)
  (parent 122)
  (seen 561)
  (unrealized (0 1))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 333)
  (parent 122)
  (seen 562)
  (unrealized (0 1))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 334)
  (parent 122)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 335)
  (parent 122)
  (seen 868)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 336)
  (parent 122)
  (unrealized (0 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 337)
  (parent 123)
  (seen 52)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 338)
  (parent 123)
  (seen 123)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 339)
  (parent 123)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 340)
  (parent 123)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 341)
  (parent 123)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 342)
  (parent 124)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 343)
  (parent 124)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 345)
  (parent 125)
  (seen 138)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 346)
  (parent 125)
  (seen 877)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 347)
  (parent 125)
  (seen 125)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 348)
  (parent 125)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 349)
  (parent 125)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 350)
  (parent 126)
  (seen 595)
  (unrealized (0 1))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 351)
  (parent 126)
  (seen 596)
  (unrealized (0 1))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 352)
  (parent 126)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 353)
  (parent 126)
  (unrealized (0 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 354)
  (parent 127)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 355)
  (parent 127)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 357)
  (parent 128)
  (seen 25)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 358)
  (parent 128)
  (seen 359)
  (unrealized (0 1) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 359)
  (parent 128)
  (seen 601)
  (unrealized (0 1))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 360)
  (parent 128)
  (unrealized (0 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 361)
  (parent 129)
  (seen 937)
  (unrealized (0 1) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 362)
  (parent 129)
  (unrealized (0 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 363)
  (parent 130)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d)))))
  (label 365)
  (parent 131)
  (seen 52)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 366)
  (parent 131)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 367)
  (parent 131)
  (seen 131)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 368)
  (parent 131)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 369)
  (parent 132)
  (seen 68 180 462)
  (unrealized (0 1) (3 0) (6 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 371)
  (parent 134)
  (seen 401)
  (unrealized (0 1) (3 0) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 373)
  (parent 135)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 377)
  (parent 136)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 380)
  (parent 136)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 383)
  (parent 140)
  (seen 16)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 384)
  (parent 140)
  (unrealized (0 1) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 385)
  (parent 141)
  (seen 58)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 386)
  (parent 141)
  (unrealized (0 1) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nc nd nc-0 nb text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c)))))
  (label 387)
  (parent 142)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 388)
  (parent 142)
  (seen 142)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 389)
  (parent 142)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 390)
  (parent 142)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 391)
  (parent 143)
  (seen 39)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 392)
  (parent 143)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 393)
  (parent 143)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 394)
  (parent 143)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 395)
  (parent 143)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 396)
  (parent 144)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 397)
  (parent 144)
  (seen 142)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nd-1 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 399)
  (parent 145)
  (unrealized (0 1) (4 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nd-1 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 400)
  (parent 145)
  (unrealized (0 1) (4 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 401)
  (parent 150)
  (unrealized (0 1) (3 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 402)
  (parent 150)
  (seen 16)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 403)
  (parent 150)
  (seen 62 970)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 404)
  (parent 150)
  (unrealized (0 1) (3 0) (5 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nc nd nb text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c)))))
  (label 406)
  (parent 152)
  (seen 27)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 407)
  (parent 152)
  (seen 332 350)
  (unrealized (0 1) (5 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 408)
  (parent 152)
  (unrealized (0 1) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 409)
  (parent 154)
  (seen 837)
  (unrealized (0 1) (3 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 410)
  (parent 154)
  (seen 842)
  (unrealized (0 1) (3 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nd nc nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 412)
  (parent 155)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 413)
  (parent 155)
  (seen 62)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 414)
  (parent 155)
  (seen 155)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 415)
  (parent 155)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 416)
  (parent 156)
  (unrealized (0 1) (3 0) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 417)
  (parent 156)
  (unrealized (0 1) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 419)
  (parent 157)
  (seen 52)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 420)
  (parent 157)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 421)
  (parent 157)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 422)
  (parent 157)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 423)
  (parent 157)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 424)
  (parent 158)
  (unrealized (0 1) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 425)
  (parent 158)
  (seen 66)
  (unrealized (0 1) (3 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 427)
  (parent 159)
  (unrealized (0 1) (3 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 428)
  (parent 159)
  (seen 1082)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 430)
  (parent 160)
  (unrealized (0 1) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 431)
  (parent 160)
  (unrealized (0 1) (3 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 433)
  (parent 160)
  (seen 498)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (4 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 435)
  (parent 161)
  (unrealized (0 1) (3 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nd nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d)))))
  (label 439)
  (parent 163)
  (seen 406)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 440)
  (parent 163)
  (seen 163)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 441)
  (parent 163)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 442)
  (parent 165)
  (unrealized (0 1) (3 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 443)
  (parent 165)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nd nb nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d)))))
  (label 445)
  (parent 166)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 446)
  (parent 166)
  (seen 166)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 447)
  (parent 166)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0) (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 448)
  (parent 167)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 451)
  (parent 167)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 454)
  (parent 168)
  (seen 69)
  (unrealized (0 1) (3 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 455)
  (parent 168)
  (seen 1057 1098)
  (unrealized (0 1) (3 0))
  (comment "9 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 457)
  (parent 169)
  (seen 52)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 458)
  (parent 169)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 459)
  (parent 169)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 460)
  (parent 169)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 461)
  (parent 169)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 462)
  (parent 170)
  (unrealized (0 1) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 463)
  (parent 170)
  (seen 1052)
  (unrealized (0 1) (3 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 465)
  (parent 171)
  (unrealized (0 1) (3 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 466)
  (parent 171)
  (seen 1171)
  (unrealized (0 1) (3 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 468)
  (parent 172)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 470)
  (parent 172)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 473)
  (parent 173)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 475)
  (parent 173)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 478)
  (parent 175)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 479)
  (parent 175)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 483)
  (parent 177)
  (seen 52)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 484)
  (parent 177)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 485)
  (parent 177)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 486)
  (parent 177)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 487)
  (parent 177)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 488)
  (parent 179)
  (unrealized (0 1) (5 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 489)
  (parent 179)
  (seen 1222)
  (unrealized (0 1) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (5 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 491)
  (parent 180)
  (seen 1088)
  (unrealized (0 1) (3 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) na (5 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 493)
  (parent 180)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 496)
  (parent 181)
  (seen 1109)
  (unrealized (0 1) (3 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 497)
  (parent 181)
  (seen 166 1098)
  (unrealized (0 1) (3 0))
  (comment "9 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 498)
  (parent 181)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nd nc nb text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c)))))
  (label 500)
  (parent 183)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 504)
  (parent 186)
  (seen 688 692)
  (unrealized (0 1) (4 0))
  (comment "8 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d)))))
  (label 507)
  (parent 187)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 513)
  (parent 189)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 517)
  (parent 189)
  (seen 782 783)
  (unrealized (0 1))
  (comment "9 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 520)
  (parent 190)
  (seen 327 328 688)
  (unrealized (0 1) (4 0))
  (comment "8 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 521)
  (parent 190)
  (seen 686 688)
  (unrealized (0 1) (4 0))
  (comment "8 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-3 nb) (nc-0 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 523)
  (parent 193)
  (seen 81)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 529)
  (parent 194)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 530)
  (parent 194)
  (seen 1294)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 531)
  (parent 194)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 533)
  (parent 195)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 534)
  (parent 195)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 535)
  (parent 195)
  (seen 1311)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 537)
  (parent 196)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 538)
  (parent 196)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 539)
  (parent 196)
  (seen 1335)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 541)
  (parent 197)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 542)
  (parent 197)
  (seen 1342)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 543)
  (parent 197)
  (seen 1343 1347)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nb-2 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nc-1 nc-0)) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d)))))
  (label 545)
  (parent 198)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 550)
  (parent 199)
  (seen 292 513)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 551)
  (parent 199)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (6 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 554)
  (parent 200)
  (seen 279 507)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 555)
  (parent 200)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb-0 nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb-0 nc (pubk d)))
      (send (enc b c d na nb-0 nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc (pubk d)))))
  (label 557)
  (parent 202)
  (seen 682)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 561)
  (parent 207)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 562)
  (parent 207)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 564)
  (parent 207)
  (seen 868)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d)))))
  (label 566)
  (parent 208)
  (seen 337)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 572)
  (parent 210)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 573)
  (parent 210)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 574)
  (parent 210)
  (unrealized (0 1) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 576)
  (parent 210)
  (seen 1412)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 578)
  (parent 211)
  (seen 229)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 585)
  (parent 212)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 586)
  (parent 212)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 589)
  (parent 214)
  (seen 345)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 595)
  (parent 215)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 596)
  (parent 215)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 599)
  (parent 217)
  (seen 37)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 601)
  (parent 217)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nb-0 nd nb-1 nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d)))))
  (label 605)
  (parent 220)
  (seen 81)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 610)
  (parent 224)
  (seen 138)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 611)
  (parent 224)
  (seen 224)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 612)
  (parent 224)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 613)
  (parent 224)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 614)
  (parent 225)
  (seen 39)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 615)
  (parent 225)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 616)
  (parent 225)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 617)
  (parent 225)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 618)
  (parent 225)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 619)
  (parent 225)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 620)
  (parent 226)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 621)
  (parent 226)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 623)
  (parent 226)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 625)
  (parent 227)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 626)
  (parent 227)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 627)
  (parent 227)
  (unrealized (0 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp3 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 628)
  (parent 227)
  (seen 1486)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 629)
  (parent 227)
  (unrealized (0 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 630)
  (parent 228)
  (seen 224 1498)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 631)
  (parent 228)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 633)
  (parent 230)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 639)
  (parent 232)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 640)
  (parent 232)
  (seen 142 397 1002)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 642)
  (parent 233)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 643)
  (parent 233)
  (seen 16)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 645)
  (parent 234)
  (seen 89)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 646)
  (parent 234)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 648)
  (parent 236)
  (seen 224 630 1463)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 650)
  (parent 238)
  (seen 20)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 652)
  (parent 239)
  (seen 91)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nc nb nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb-0 nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb-0 nc (pubk d)))
      (send (enc b c d na nb-0 nc nd (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 654)
  (parent 240)
  (seen 387)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 659)
  (parent 241)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 660)
  (parent 241)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 661)
  (parent 241)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 664)
  (parent 242)
  (seen 391)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 670)
  (parent 244)
  (seen 1345)
  (unrealized (0 1) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 675)
  (parent 248)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 676)
  (parent 248)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 679)
  (parent 251)
  (seen 20)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 680)
  (parent 251)
  (seen 95 970)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nc nb nd nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb-0 nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb-0 nc (pubk d)))
      (send (enc b c d na nb-0 nc nd (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 682)
  (parent 254)
  (seen 406)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 683)
  (parent 254)
  (seen 561 595 1037)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 684)
  (parent 254)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-1 (pubk a)))))
  (label 685)
  (parent 254)
  (seen 1040)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 686)
  (parent 257)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 688)
  (parent 257)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 692)
  (parent 258)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 694)
  (parent 258)
  (seen 1591 1592)
  (unrealized (0 1))
  (comment "8 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nc-1 nc-0)) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 698)
  (parent 259)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 702)
  (parent 259)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nb-0 nd nc nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 705)
  (parent 262)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 710)
  (parent 265)
  (seen 419)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nc-1 nc-0)) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d)))))
  (label 716)
  (parent 268)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 720)
  (parent 269)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 721)
  (parent 269)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 724)
  (parent 269)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d)))))
  (label 726)
  (parent 270)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 730)
  (parent 270)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 733)
  (parent 272)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d)))))
  (label 737)
  (parent 274)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb-0 nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb-0 nc (pubk d)))
      (send (enc b c d nb nb-0 nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc (pubk d)))))
  (label 742)
  (parent 277)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c)))))
  (label 747)
  (parent 279)
  (seen 406)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 748)
  (parent 279)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 749)
  (parent 279)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 750)
  (parent 279)
  (seen 1293 1338)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 751)
  (parent 279)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 752)
  (parent 283)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 757)
  (parent 284)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 761)
  (parent 284)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 764)
  (parent 285)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 765)
  (parent 285)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 766)
  (parent 285)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 768)
  (parent 285)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 770)
  (parent 287)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d)))))
  (label 774)
  (parent 289)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b)))
      (send (enc a b d nb nb-0 (pubk c)))))
  (label 779)
  (parent 292)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 780)
  (parent 292)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 781)
  (parent 292)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 782)
  (parent 292)
  (seen 1309 1333)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 783)
  (parent 292)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 784)
  (parent 295)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 787)
  (parent 296)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nb-0 nd nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d)))))
  (label 792)
  (parent 298)
  (seen 682)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-1 nc-0)) na (4 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 796)
  (parent 299)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb-0 nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb-0 nc (pubk d)))
      (send (enc b c d na nb-0 nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc (pubk d)))))
  (label 800)
  (parent 302)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d)))))
  (label 804)
  (parent 305)
  (seen 457)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d)))))
  (label 810)
  (parent 308)
  (seen 483)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 816)
  (parent 314)
  (seen 115)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 817)
  (parent 314)
  (unrealized (0 1) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nd nb nc nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 819)
  (parent 315)
  (seen 972)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-2 nc-1)) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-0 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 824)
  (parent 320)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-2 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 825)
  (parent 320)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 833)
  (parent 323)
  (unrealized (0 1) (3 0) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (6 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 835)
  (parent 323)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 836)
  (parent 323)
  (seen 16)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (6 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 837)
  (parent 323)
  (seen 115)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 839)
  (parent 324)
  (seen 1777)
  (unrealized (0 1) (3 0) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 841)
  (parent 324)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 842)
  (parent 324)
  (seen 16)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (6 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 843)
  (parent 324)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nd nc nb text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c)))))
  (label 845)
  (parent 326)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 849)
  (parent 329)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 855)
  (parent 332)
  (seen 138)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 856)
  (parent 332)
  (seen 332)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 857)
  (parent 332)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 858)
  (parent 332)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 859)
  (parent 333)
  (seen 52)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 860)
  (parent 333)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 861)
  (parent 333)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 862)
  (parent 333)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 863)
  (parent 333)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 864)
  (parent 334)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 865)
  (parent 334)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 866)
  (parent 334)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 868)
  (parent 334)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 870)
  (parent 335)
  (seen 1390)
  (unrealized (0 1))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 871)
  (parent 335)
  (seen 1391)
  (unrealized (0 1))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 872)
  (parent 335)
  (unrealized (0 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 873)
  (parent 335)
  (unrealized (0 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 874)
  (parent 336)
  (seen 332 1857)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 875)
  (parent 336)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 877)
  (parent 339)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 878)
  (parent 339)
  (seen 350 922)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 880)
  (parent 340)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 881)
  (parent 340)
  (seen 1882)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 883)
  (parent 341)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 884)
  (parent 341)
  (seen 951 1890)
  (unrealized (0 1) (6 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 886)
  (parent 342)
  (seen 229)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 892)
  (parent 343)
  (seen 81)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 899)
  (parent 346)
  (seen 332 874 1814)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 901)
  (parent 348)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 902)
  (parent 348)
  (seen 1899)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 904)
  (parent 349)
  (seen 945)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 905)
  (parent 349)
  (seen 1906)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nc nd nb nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 907)
  (parent 350)
  (seen 406)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 908)
  (parent 350)
  (seen 350)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 909)
  (parent 350)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 910)
  (parent 350)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 911)
  (parent 351)
  (seen 138)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 912)
  (parent 351)
  (seen 1879)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 913)
  (parent 351)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 914)
  (parent 351)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 915)
  (parent 351)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 916)
  (parent 352)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 917)
  (parent 352)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 918)
  (parent 352)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 921)
  (parent 353)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 922)
  (parent 353)
  (seen 350)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nc nd nb nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb-0 nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc (pubk d))))
    ((recv (enc a b c na nb-0 nc (pubk d)))
      (send (enc b c d na nb-0 nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 924)
  (parent 354)
  (seen 387)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 930)
  (parent 355)
  (seen 391)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 937)
  (parent 358)
  (unrealized (0 1) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-1 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)) (enc b c d na nb nc nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd-0 (pubk a)))))
  (label 940)
  (parent 359)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-1 nd-0)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)) (enc b c d na nb nc nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd-0 (pubk a)))
      (send (enc nb nc nd-0 (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd-0 (pubk a)))))
  (label 941)
  (parent 359)
  (seen 39)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a))
    (enc b c d na nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 942)
  (parent 359)
  (seen 1892)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a))
    (enc b c d na nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 943)
  (parent 359)
  (seen 1908)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nd-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a))
    (enc b c d na nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-2 (pubk a)))))
  (label 944)
  (parent 359)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 945)
  (parent 360)
  (seen 357 358)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 946)
  (parent 360)
  (unrealized (0 1) (5 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 950)
  (parent 362)
  (seen 1973)
  (unrealized (0 1) (5 0) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 951)
  (parent 362)
  (seen 60)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-1 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b d nc nb (pubk c))
    (enc a b d nc nb-0 (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc nd (pubk a)) (enc b c d nc nb nc nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb-0 (pubk c))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd-0 (pubk a)))))
  (label 953)
  (parent 363)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-1 nd-0)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b d nc nb (pubk c))
    (enc a b d nc nb-0 (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc nd (pubk a)) (enc b c d nc nb nc nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd-0 (pubk a)))
      (send (enc nb nc nd-0 (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb-0 (pubk c))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd-0 (pubk a)))))
  (label 954)
  (parent 363)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))
    (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 955)
  (parent 363)
  (unrealized (0 1))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))
    (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 956)
  (parent 363)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))
    (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 957)
  (parent 363)
  (unrealized (0 1) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(comment "Step limit exceeded--aborting run")

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nd-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))
    (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-2 (pubk a)))))
  (label 958)
  (parent 363)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 959)
  (parent 366)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 960)
  (parent 366)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 961)
  (parent 366)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 962)
  (parent 368)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 963)
  (parent 368)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 964)
  (parent 368)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 965)
  (parent 369)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 966)
  (parent 369)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 967)
  (parent 369)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 968)
  (parent 371)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 969)
  (parent 371)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 970)
  (parent 371)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 971)
  (parent 371)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nc nd nb nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb-0 nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc (pubk d))))
    ((recv (enc a b c na nb-0 nc (pubk d)))
      (send (enc b c d na nb-0 nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 972)
  (parent 373)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 973)
  (parent 373)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 974)
  (parent 373)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 975)
  (parent 373)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-1 (pubk a)))))
  (label 976)
  (parent 373)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nd nb-0 nc nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 977)
  (parent 377)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 nb-3 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 978)
  (parent 377)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 979)
  (parent 377)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 980)
  (parent 377)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 981)
  (parent 377)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 982)
  (parent 377)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 983)
  (parent 380)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 984)
  (parent 380)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 985)
  (parent 380)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 986)
  (parent 380)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 987)
  (parent 380)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 988)
  (parent 380)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 989)
  (parent 380)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 990)
  (parent 384)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 991)
  (parent 384)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 992)
  (parent 386)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 993)
  (parent 386)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nc nd nb text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation generalization deleted (4 0))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c)))))
  (label 994)
  (parent 387)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 995)
  (parent 389)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 996)
  (parent 389)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 997)
  (parent 389)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 998)
  (parent 390)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 999)
  (parent 390)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1000)
  (parent 390)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1001)
  (parent 392)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1002)
  (parent 392)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1003)
  (parent 392)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1004)
  (parent 393)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1005)
  (parent 393)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1006)
  (parent 393)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1007)
  (parent 394)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1008)
  (parent 394)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1009)
  (parent 394)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1010)
  (parent 395)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1011)
  (parent 395)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1012)
  (parent 395)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1013)
  (parent 396)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1014)
  (parent 396)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1015)
  (parent 396)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1016)
  (parent 396)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1017)
  (parent 396)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (4 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1018)
  (parent 397)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1019)
  (parent 397)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1020)
  (parent 397)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-2 (pubk d)))
      (send (enc b c d na nb-1 nc-2 nd-1 (pubk a)))))
  (label 1021)
  (parent 397)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nd-1 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1022)
  (parent 399)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nd-1 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1023)
  (parent 399)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nd-1 nb-1 nc-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1024)
  (parent 399)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nd-1 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1025)
  (parent 400)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nd-1 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1026)
  (parent 400)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nd-1 nb-1 nc-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1027)
  (parent 400)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1028)
  (parent 401)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1029)
  (parent 401)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1030)
  (parent 401)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1031)
  (parent 401)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1032)
  (parent 403)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1033)
  (parent 403)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nd-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-2 (pubk a)))))
  (label 1034)
  (parent 403)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1035)
  (parent 404)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nc-2 nb-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (7 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1036)
  (parent 404)
  (unrealized (0 1) (3 0) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1037)
  (parent 407)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1038)
  (parent 408)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1039)
  (parent 408)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1040)
  (parent 408)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1041)
  (parent 409)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1042)
  (parent 409)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1043)
  (parent 409)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1044)
  (parent 410)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1045)
  (parent 410)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1046)
  (parent 410)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nd nc nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation generalization deleted (2 0))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1047)
  (parent 412)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1048)
  (parent 415)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1049)
  (parent 415)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1050)
  (parent 415)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1051)
  (parent 416)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1052)
  (parent 416)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1053)
  (parent 416)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1054)
  (parent 416)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1055)
  (parent 416)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-2 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1056)
  (parent 417)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-2 nc-1)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1057)
  (parent 417)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1058)
  (parent 417)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1059)
  (parent 417)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1060)
  (parent 417)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-2 (pubk d)))
      (send (enc b c d na nb-1 nc-2 nd-1 (pubk a)))))
  (label 1061)
  (parent 417)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1062)
  (parent 417)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1063)
  (parent 420)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1064)
  (parent 420)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1065)
  (parent 420)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1066)
  (parent 421)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1067)
  (parent 421)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1068)
  (parent 421)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1069)
  (parent 422)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1070)
  (parent 422)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1071)
  (parent 422)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1072)
  (parent 423)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1073)
  (parent 423)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1074)
  (parent 423)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-2 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1075)
  (parent 424)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-2 nc-1)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1076)
  (parent 424)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1077)
  (parent 424)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1078)
  (parent 424)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1079)
  (parent 424)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-1 (pubk a)))))
  (label 1080)
  (parent 424)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1081)
  (parent 424)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1082)
  (parent 427)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1083)
  (parent 427)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1084)
  (parent 427)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1085)
  (parent 427)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1086)
  (parent 428)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1087)
  (parent 428)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-2 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1088)
  (parent 430)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-2 nc-1)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1089)
  (parent 430)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1090)
  (parent 430)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1091)
  (parent 430)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1092)
  (parent 430)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-1 (pubk a)))))
  (label 1093)
  (parent 430)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1094)
  (parent 430)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-2 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1095)
  (parent 431)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-2 nc-1)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1096)
  (parent 431)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1097)
  (parent 431)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1098)
  (parent 431)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1099)
  (parent 431)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1100)
  (parent 431)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1101)
  (parent 431)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-2 (pubk d)))
      (send (enc b c d na nb-1 nc-2 nd-1 (pubk a)))))
  (label 1102)
  (parent 431)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1103)
  (parent 431)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1104)
  (parent 433)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1105)
  (parent 433)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1106)
  (parent 433)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1107)
  (parent 433)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-2 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1108)
  (parent 435)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-2 nc-1)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1109)
  (parent 435)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1110)
  (parent 435)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1111)
  (parent 435)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1112)
  (parent 435)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1113)
  (parent 435)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-2 (pubk d)))
      (send (enc b c d na nb-1 nc-2 nd-1 (pubk a)))))
  (label 1114)
  (parent 435)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1115)
  (parent 435)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1116)
  (parent 441)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1117)
  (parent 441)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1118)
  (parent 441)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1119)
  (parent 442)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1120)
  (parent 442)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1121)
  (parent 442)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1122)
  (parent 442)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1123)
  (parent 443)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (6 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1124)
  (parent 443)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1125)
  (parent 443)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nd nb nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation generalization deleted (2 0))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d)))))
  (label 1126)
  (parent 445)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1127)
  (parent 447)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1128)
  (parent 447)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1129)
  (parent 447)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nd nb nc nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1130)
  (parent 448)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1131)
  (parent 448)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1132)
  (parent 448)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1133)
  (parent 448)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1134)
  (parent 448)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1135)
  (parent 451)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 1136)
  (parent 451)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1137)
  (parent 451)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1138)
  (parent 451)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1139)
  (parent 451)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1140)
  (parent 451)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1141)
  (parent 451)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-2 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1142)
  (parent 455)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1143)
  (parent 455)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1144)
  (parent 455)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1145)
  (parent 455)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1146)
  (parent 455)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-2 (pubk d)))
      (send (enc b c d na nb-1 nc-2 nd-1 (pubk a)))))
  (label 1147)
  (parent 455)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1148)
  (parent 455)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1149)
  (parent 458)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1150)
  (parent 458)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1151)
  (parent 458)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1152)
  (parent 459)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1153)
  (parent 459)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1154)
  (parent 459)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1155)
  (parent 460)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1156)
  (parent 460)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1157)
  (parent 460)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1158)
  (parent 461)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1159)
  (parent 461)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1160)
  (parent 461)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-2 nc-1)) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-0 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1161)
  (parent 462)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-2 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1162)
  (parent 462)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1163)
  (parent 462)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1164)
  (parent 462)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1165)
  (parent 462)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1166)
  (parent 462)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-1 (pubk a)))))
  (label 1167)
  (parent 462)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1168)
  (parent 463)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1169)
  (parent 463)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1170)
  (parent 463)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1171)
  (parent 465)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1172)
  (parent 465)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1173)
  (parent 465)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1174)
  (parent 465)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1175)
  (parent 466)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1176)
  (parent 466)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nd nc nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1177)
  (parent 468)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1178)
  (parent 468)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1179)
  (parent 468)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1180)
  (parent 468)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1181)
  (parent 468)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1182)
  (parent 470)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1183)
  (parent 470)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1184)
  (parent 470)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1185)
  (parent 470)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1186)
  (parent 470)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1187)
  (parent 470)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nd nc nb text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c)))))
  (label 1188)
  (parent 473)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1189)
  (parent 473)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1190)
  (parent 473)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1191)
  (parent 473)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1192)
  (parent 475)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1193)
  (parent 475)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1194)
  (parent 475)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1195)
  (parent 475)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1196)
  (parent 475)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1197)
  (parent 475)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1198)
  (parent 478)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1199)
  (parent 478)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1200)
  (parent 478)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1201)
  (parent 478)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1202)
  (parent 478)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1203)
  (parent 478)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1204)
  (parent 479)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1205)
  (parent 479)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1206)
  (parent 479)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1207)
  (parent 479)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1208)
  (parent 479)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1209)
  (parent 479)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1210)
  (parent 484)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1211)
  (parent 484)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1212)
  (parent 484)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1213)
  (parent 485)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1214)
  (parent 485)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1215)
  (parent 485)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1216)
  (parent 486)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1217)
  (parent 486)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1218)
  (parent 486)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1219)
  (parent 487)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1220)
  (parent 487)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1221)
  (parent 487)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1222)
  (parent 488)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1223)
  (parent 488)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1224)
  (parent 488)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1225)
  (parent 488)
  (unrealized (0 1) (5 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1226)
  (parent 489)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-2 nc-1)) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-0 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1227)
  (parent 491)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1228)
  (parent 491)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1229)
  (parent 491)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1230)
  (parent 491)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1231)
  (parent 491)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1232)
  (parent 491)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-1 (pubk a)))))
  (label 1233)
  (parent 491)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-1 nb-0)) nc-0 (5 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-2 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1234)
  (parent 493)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 4 resp1 2) nc-0 (5 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-2 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1235)
  (parent 493)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-2 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 1236)
  (parent 493)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-2 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-3 (pubk d)))))
  (label 1237)
  (parent 493)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (5 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-2 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-2 nd-1 (pubk a)))))
  (label 1238)
  (parent 493)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (5 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-2 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1239)
  (parent 493)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-2 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1240)
  (parent 496)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1241)
  (parent 496)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1242)
  (parent 496)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1243)
  (parent 496)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nc-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-1 (pubk a)))))
  (label 1244)
  (parent 496)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nc-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1245)
  (parent 496)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-2 nc-1)) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-0 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1246)
  (parent 497)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1247)
  (parent 497)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1248)
  (parent 497)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1249)
  (parent 497)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1250)
  (parent 497)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1251)
  (parent 497)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-1 (pubk a)))))
  (label 1252)
  (parent 497)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 1253)
  (parent 498)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1254)
  (parent 498)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1255)
  (parent 498)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1256)
  (parent 498)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1257)
  (parent 498)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nd nc nb text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation generalization deleted (2 0))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c)))))
  (label 1258)
  (parent 500)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1259)
  (parent 504)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1260)
  (parent 504)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1261)
  (parent 504)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 1262)
  (parent 504)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1263)
  (parent 504)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-2 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-2 nc-1 nd-1 (pubk a)))))
  (label 1264)
  (parent 504)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb-0 nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb-0 nc (pubk d)))
      (send (enc b c d nb nb-0 nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc (pubk d)))))
  (label 1265)
  (parent 507)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1266)
  (parent 507)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1267)
  (parent 507)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1268)
  (parent 507)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-1 (pubk a)))))
  (label 1269)
  (parent 507)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d)))))
  (label 1270)
  (parent 513)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1271)
  (parent 513)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1272)
  (parent 513)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1273)
  (parent 513)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1274)
  (parent 513)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d))
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 1275)
  (parent 517)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1276)
  (parent 517)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1277)
  (parent 517)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1278)
  (parent 517)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1279)
  (parent 517)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1280)
  (parent 517)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1281)
  (parent 517)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 1282)
  (parent 520)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1283)
  (parent 520)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1284)
  (parent 520)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1285)
  (parent 520)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1286)
  (parent 520)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1287)
  (parent 521)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1288)
  (parent 521)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1289)
  (parent 521)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1290)
  (parent 521)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1291)
  (parent 521)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1292)
  (parent 521)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1293)
  (parent 529)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 1294)
  (parent 529)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 1295)
  (parent 529)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1296)
  (parent 529)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 1297)
  (parent 529)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1298)
  (parent 529)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 1299)
  (parent 530)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1300)
  (parent 530)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1301)
  (parent 530)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d))
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d)))))
  (label 1302)
  (parent 531)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1303)
  (parent 531)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1304)
  (parent 531)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1305)
  (parent 531)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1306)
  (parent 531)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-2 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-2 nc-1 nd-1 (pubk a)))))
  (label 1307)
  (parent 531)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1308)
  (parent 531)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1309)
  (parent 533)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1310)
  (parent 533)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1311)
  (parent 533)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1312)
  (parent 533)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1313)
  (parent 533)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1314)
  (parent 533)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d))
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 1315)
  (parent 534)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1316)
  (parent 534)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1317)
  (parent 534)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1318)
  (parent 534)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1319)
  (parent 534)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1320)
  (parent 534)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1321)
  (parent 534)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1322)
  (parent 535)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1323)
  (parent 535)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1324)
  (parent 535)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1325)
  (parent 535)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d))
    (enc a b c nb-0 nb nc-0 (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc-0 (pubk d)))))
  (label 1326)
  (parent 537)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1327)
  (parent 537)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1328)
  (parent 537)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1329)
  (parent 537)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1330)
  (parent 537)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1331)
  (parent 537)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1332)
  (parent 537)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1333)
  (parent 538)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1334)
  (parent 538)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1335)
  (parent 538)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1336)
  (parent 538)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1337)
  (parent 538)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (6 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1338)
  (parent 539)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1339)
  (parent 539)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1340)
  (parent 539)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1341)
  (parent 539)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1342)
  (parent 541)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1343)
  (parent 541)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1344)
  (parent 541)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1345)
  (parent 541)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1346)
  (parent 541)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1347)
  (parent 542)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1348)
  (parent 542)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1349)
  (parent 542)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1350)
  (parent 543)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 1351)
  (parent 543)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nb-0 nd nb-1 nc text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-0 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d)))))
  (label 1352)
  (parent 545)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nb-2 nc-0 nb-3 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1353)
  (parent 545)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nb-2 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 1354)
  (parent 545)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nb-2 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1355)
  (parent 545)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nb-2 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1356)
  (parent 545)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nb-2 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1357)
  (parent 545)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nc-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nc-1 (pubk c)))))
  (label 1358)
  (parent 550)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1359)
  (parent 550)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1360)
  (parent 550)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1361)
  (parent 550)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nb-2 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nc-1 nc-0)) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d)))))
  (label 1362)
  (parent 551)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1363)
  (parent 551)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1364)
  (parent 551)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1365)
  (parent 551)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1366)
  (parent 551)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1367)
  (parent 551)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1368)
  (parent 554)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1369)
  (parent 554)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1370)
  (parent 554)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1371)
  (parent 554)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nb-2 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nc-1 nc-0)) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d)))))
  (label 1372)
  (parent 555)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1373)
  (parent 555)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1374)
  (parent 555)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1375)
  (parent 555)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1376)
  (parent 555)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1377)
  (parent 555)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1378)
  (parent 555)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 1379)
  (parent 561)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1380)
  (parent 561)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1381)
  (parent 561)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1382)
  (parent 561)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1383)
  (parent 561)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d)))))
  (label 1384)
  (parent 562)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1385)
  (parent 562)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1386)
  (parent 562)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1387)
  (parent 562)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b c na nb-2 nc-1 (pubk d)))
      (send (enc b c d na nb-2 nc-1 nd-1 (pubk a)))))
  (label 1388)
  (parent 562)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1389)
  (parent 562)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1390)
  (parent 564)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 1391)
  (parent 564)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 1392)
  (parent 564)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1393)
  (parent 564)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1394)
  (parent 564)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 1395)
  (parent 572)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1396)
  (parent 572)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1397)
  (parent 572)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1398)
  (parent 572)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1399)
  (parent 572)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 1400)
  (parent 573)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 1401)
  (parent 573)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1402)
  (parent 573)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1403)
  (parent 573)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1404)
  (parent 573)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1405)
  (parent 573)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1406)
  (parent 573)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1407)
  (parent 574)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1408)
  (parent 574)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1409)
  (parent 574)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 1410)
  (parent 574)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1411)
  (parent 574)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1412)
  (parent 574)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1413)
  (parent 574)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1414)
  (parent 576)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1415)
  (parent 576)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 1416)
  (parent 576)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1417)
  (parent 576)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1418)
  (parent 576)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 1419)
  (parent 585)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1420)
  (parent 585)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1421)
  (parent 585)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1422)
  (parent 585)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1423)
  (parent 585)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 1424)
  (parent 586)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 1425)
  (parent 586)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1426)
  (parent 586)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1427)
  (parent 586)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1428)
  (parent 586)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1429)
  (parent 586)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1430)
  (parent 586)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nc nb nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb-0 nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb-0 nc (pubk d)))
      (send (enc b c d na nb-0 nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 1431)
  (parent 595)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1432)
  (parent 595)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1433)
  (parent 595)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1434)
  (parent 595)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-1 (pubk a)))))
  (label 1435)
  (parent 595)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 1436)
  (parent 596)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1437)
  (parent 596)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1438)
  (parent 596)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1439)
  (parent 596)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1440)
  (parent 596)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1441)
  (parent 596)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-0 nc) (nd-1 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a))
    (enc b c d na nb nc nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd-0 (pubk a)))))
  (label 1442)
  (parent 601)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-0 nc) (nd-1 nd-0)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a))
    (enc b c d na nb nc nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd-0 (pubk a)))
      (send (enc nb nc nd-0 (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd-0 (pubk a)))))
  (label 1443)
  (parent 601)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a))
    (enc b c d na nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 1444)
  (parent 601)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a))
    (enc b c d na nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1445)
  (parent 601)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a))
    (enc b c d na nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1446)
  (parent 601)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nd-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a))
    (enc b c d na nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-2 (pubk a)))))
  (label 1447)
  (parent 601)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1448)
  (parent 612)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1449)
  (parent 612)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1450)
  (parent 612)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1451)
  (parent 613)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1452)
  (parent 613)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1453)
  (parent 613)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1454)
  (parent 615)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1455)
  (parent 615)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1456)
  (parent 615)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1457)
  (parent 615)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1458)
  (parent 615)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1459)
  (parent 615)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1460)
  (parent 616)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1461)
  (parent 616)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1462)
  (parent 616)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1463)
  (parent 617)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1464)
  (parent 617)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1465)
  (parent 617)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1466)
  (parent 618)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1467)
  (parent 618)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1468)
  (parent 618)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1469)
  (parent 619)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1470)
  (parent 619)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1471)
  (parent 619)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1472)
  (parent 620)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1473)
  (parent 620)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1474)
  (parent 620)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1475)
  (parent 620)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1476)
  (parent 620)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1477)
  (parent 621)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1478)
  (parent 621)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1479)
  (parent 621)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1480)
  (parent 621)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1481)
  (parent 621)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1482)
  (parent 623)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1483)
  (parent 623)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1484)
  (parent 623)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1485)
  (parent 623)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp3 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1486)
  (parent 623)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1487)
  (parent 623)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 1488)
  (parent 625)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1489)
  (parent 625)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1490)
  (parent 625)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1491)
  (parent 625)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 1492)
  (parent 626)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1493)
  (parent 626)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1494)
  (parent 626)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1495)
  (parent 626)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1496)
  (parent 626)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1497)
  (parent 626)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1498)
  (parent 627)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1499)
  (parent 627)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1500)
  (parent 627)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1501)
  (parent 628)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1502)
  (parent 628)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1503)
  (parent 628)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1504)
  (parent 628)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1505)
  (parent 629)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1506)
  (parent 629)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1507)
  (parent 629)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (5 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1508)
  (parent 630)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1509)
  (parent 630)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1510)
  (parent 630)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-1 (pubk a)))))
  (label 1511)
  (parent 630)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1512)
  (parent 631)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1513)
  (parent 631)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1514)
  (parent 631)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1515)
  (parent 631)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation generalization deleted (5 0))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1516)
  (parent 633)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nb-0 nd nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-2 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-1 (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-1 (pubk d)))))
  (label 1517)
  (parent 639)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1518)
  (parent 639)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1519)
  (parent 639)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1520)
  (parent 639)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1521)
  (parent 639)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-1 (pubk a)))))
  (label 1522)
  (parent 639)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1523)
  (parent 639)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1524)
  (parent 640)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1525)
  (parent 640)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1526)
  (parent 642)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1527)
  (parent 642)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (6 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1528)
  (parent 642)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1529)
  (parent 642)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1530)
  (parent 646)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1531)
  (parent 646)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1532)
  (parent 646)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1533)
  (parent 646)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1534)
  (parent 646)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1535)
  (parent 648)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1536)
  (parent 648)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1537)
  (parent 648)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b c nb nb nc-0 (pubk d))
    (enc a b d nb nb (pubk c)) (enc a b d nb nb-0 (pubk c))
    (enc a c d nb (pubk b)) (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc-0 (pubk d))))
    ((recv (enc a c d nb (pubk b)))
      (send (enc a b d nb nb-0 (pubk c)))))
  (label 1538)
  (parent 659)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1539)
  (parent 659)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1540)
  (parent 659)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1541)
  (parent 659)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1542)
  (parent 659)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1543)
  (parent 659)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb-0 nc (pubk d))
    (enc a b c nb nb-0 nc-0 (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb-0 nc (pubk d)))
      (send (enc b c d nb nb-0 nc nd (pubk a))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb (pubk b)))
      (send (enc a b d nb nb-0 (pubk c)))))
  (label 1544)
  (parent 660)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1545)
  (parent 660)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1546)
  (parent 660)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1547)
  (parent 660)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1548)
  (parent 660)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-1 (pubk a)))))
  (label 1549)
  (parent 660)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d))
    (enc a b c nb-0 nb nc-0 (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1550)
  (parent 661)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1551)
  (parent 661)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1552)
  (parent 661)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1553)
  (parent 661)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1554)
  (parent 661)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1555)
  (parent 661)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1556)
  (parent 661)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 1557)
  (parent 670)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1558)
  (parent 670)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1559)
  (parent 670)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 1560)
  (parent 675)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1561)
  (parent 675)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1562)
  (parent 675)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1563)
  (parent 675)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1564)
  (parent 675)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1565)
  (parent 675)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nb-0 nd nc nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 1566)
  (parent 676)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1567)
  (parent 676)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1568)
  (parent 676)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1569)
  (parent 676)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1570)
  (parent 676)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1571)
  (parent 676)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1572)
  (parent 680)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1573)
  (parent 680)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1574)
  (parent 680)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nd-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-2 (pubk a)))))
  (label 1575)
  (parent 680)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1576)
  (parent 683)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1577)
  (parent 684)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1578)
  (parent 684)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1579)
  (parent 684)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1580)
  (parent 684)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-1 (pubk a)))))
  (label 1581)
  (parent 685)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-1 (pubk a)))))
  (label 1582)
  (parent 685)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nb-1 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1583)
  (parent 685)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nc nb-0 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b)))
      (send (enc a b d nb nb-0 (pubk c)))))
  (label 1584)
  (parent 686)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1585)
  (parent 686)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1586)
  (parent 686)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1587)
  (parent 686)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1588)
  (parent 686)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nd text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a c d nb (pubk b)) (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a)))))
  (label 1589)
  (parent 688)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1590)
  (parent 688)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1591)
  (parent 688)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1592)
  (parent 688)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nc nb-0 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb-0 nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb-0 nc (pubk d)))
      (send (enc b c d nb nb-0 nc nd (pubk a))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc (pubk d))))
    ((recv (enc a c d nb (pubk b)))
      (send (enc a b d nb nb-0 (pubk c)))))
  (label 1593)
  (parent 692)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1594)
  (parent 692)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1595)
  (parent 692)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1596)
  (parent 692)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-1 (pubk a)))))
  (label 1597)
  (parent 692)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb-0 nc (pubk d))
    (enc a b c nb nb-0 nc-0 (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb-0 nc (pubk d)))
      (send (enc b c d nb nb-0 nc nd (pubk a))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb (pubk b)))
      (send (enc a b d nb nb-0 (pubk c)))))
  (label 1598)
  (parent 694)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1599)
  (parent 694)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1600)
  (parent 694)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1601)
  (parent 694)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1602)
  (parent 694)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-1 (pubk a)))))
  (label 1603)
  (parent 694)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nb-0 nd nc nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-0 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1604)
  (parent 698)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nc-0 nb-2 nb-3 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1605)
  (parent 698)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nc-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 1606)
  (parent 698)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nc-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1607)
  (parent 698)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nc-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1608)
  (parent 698)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nc-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1609)
  (parent 698)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d))
    (enc a b c nb-0 nb nc-0 (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1610)
  (parent 702)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1611)
  (parent 702)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1612)
  (parent 702)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1613)
  (parent 702)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1614)
  (parent 702)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1615)
  (parent 702)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1616)
  (parent 702)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nb-0 nd nc nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation generalization deleted (2 0))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1617)
  (parent 705)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nb-0 nd nc text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d)))))
  (label 1618)
  (parent 716)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1619)
  (parent 716)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1620)
  (parent 716)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1621)
  (parent 716)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1622)
  (parent 716)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nc-1 nc-0)) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d)))))
  (label 1623)
  (parent 720)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1624)
  (parent 720)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1625)
  (parent 720)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1626)
  (parent 720)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1627)
  (parent 720)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d)))))
  (label 1628)
  (parent 721)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1629)
  (parent 721)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1630)
  (parent 721)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1631)
  (parent 721)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 1632)
  (parent 721)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1633)
  (parent 721)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-2 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-2 nc-1 nd-1 (pubk a)))))
  (label 1634)
  (parent 721)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1635)
  (parent 724)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 1636)
  (parent 724)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1637)
  (parent 724)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1638)
  (parent 724)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1639)
  (parent 724)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1640)
  (parent 724)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb-0 nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb-0 nc (pubk d)))
      (send (enc b c d nb nb-0 nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc (pubk d)))))
  (label 1641)
  (parent 726)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1642)
  (parent 726)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1643)
  (parent 726)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1644)
  (parent 726)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-1 (pubk a)))))
  (label 1645)
  (parent 726)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d))
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d)))))
  (label 1646)
  (parent 730)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1647)
  (parent 730)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1648)
  (parent 730)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1649)
  (parent 730)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1650)
  (parent 730)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-2 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-2 nc-1 nd-1 (pubk a)))))
  (label 1651)
  (parent 730)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1652)
  (parent 730)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d))
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d)))))
  (label 1653)
  (parent 733)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1654)
  (parent 733)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1655)
  (parent 733)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1656)
  (parent 733)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1657)
  (parent 733)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-2 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-2 nc-1 nd-1 (pubk a)))))
  (label 1658)
  (parent 733)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1659)
  (parent 733)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb-0 nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb-0 nc (pubk d)))
      (send (enc b c d nb nb-0 nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc (pubk d)))))
  (label 1660)
  (parent 737)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1661)
  (parent 737)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1662)
  (parent 737)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1663)
  (parent 737)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-1 (pubk a)))))
  (label 1664)
  (parent 737)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation generalization deleted (2 0))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb-0 nc (pubk d)))
      (send (enc b c d nb nb-0 nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nc (pubk d)))))
  (label 1665)
  (parent 742)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-0 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1666)
  (parent 748)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 1667)
  (parent 748)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1668)
  (parent 748)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1669)
  (parent 748)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1670)
  (parent 748)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1671)
  (parent 749)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1672)
  (parent 749)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1673)
  (parent 749)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1674)
  (parent 749)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 1675)
  (parent 750)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1676)
  (parent 750)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1677)
  (parent 751)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1678)
  (parent 751)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1679)
  (parent 751)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1680)
  (parent 751)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d)))))
  (label 1681)
  (parent 752)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1682)
  (parent 752)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1683)
  (parent 752)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1684)
  (parent 752)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1685)
  (parent 752)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d)))))
  (label 1686)
  (parent 757)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1687)
  (parent 757)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1688)
  (parent 757)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1689)
  (parent 757)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1690)
  (parent 757)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d))
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 1691)
  (parent 761)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1692)
  (parent 761)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1693)
  (parent 761)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1694)
  (parent 761)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1695)
  (parent 761)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1696)
  (parent 761)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1697)
  (parent 761)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 1698)
  (parent 764)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1699)
  (parent 764)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1700)
  (parent 764)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1701)
  (parent 764)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1702)
  (parent 764)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1703)
  (parent 764)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 1704)
  (parent 765)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1705)
  (parent 765)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1706)
  (parent 765)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1707)
  (parent 765)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1708)
  (parent 765)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1709)
  (parent 765)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 1710)
  (parent 766)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1711)
  (parent 766)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1712)
  (parent 766)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1713)
  (parent 766)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1714)
  (parent 766)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1715)
  (parent 766)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1716)
  (parent 766)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b c nb-2 nb-0 nc-0 (pubk d)) (enc a b d nb-2 nb-0 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1717)
  (parent 768)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1718)
  (parent 768)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-0 (6 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1719)
  (parent 768)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1720)
  (parent 768)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1721)
  (parent 768)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1722)
  (parent 768)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) nb-0
    (0 1) (enc a b c nb-0 nb nc (pubk d))
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nc (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb nc (pubk d)))
      (send (enc b c d nb-0 nb nc nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 1723)
  (parent 770)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1724)
  (parent 770)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1725)
  (parent 770)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1726)
  (parent 770)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1727)
  (parent 770)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1728)
  (parent 770)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1729)
  (parent 770)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation generalization deleted (2 0))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d)))))
  (label 1730)
  (parent 774)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation generalization deleted (3 0))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b)))
      (send (enc a b d nb nb-0 (pubk c)))))
  (label 1731)
  (parent 779)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-2 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a b d nb nb-1 (pubk c))
    (enc a c d nb (pubk b)) (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a c d nb (pubk b)))
      (send (enc a b d nb nb-1 (pubk c)))))
  (label 1732)
  (parent 780)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nc-0 (pubk c)))))
  (label 1733)
  (parent 780)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 1734)
  (parent 780)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1735)
  (parent 780)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1736)
  (parent 780)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1737)
  (parent 780)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1738)
  (parent 781)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1739)
  (parent 781)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1740)
  (parent 781)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1741)
  (parent 781)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1742)
  (parent 782)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1743)
  (parent 782)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1744)
  (parent 783)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1745)
  (parent 783)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1746)
  (parent 783)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1747)
  (parent 783)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nd nb nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a c d nb (pubk b)) (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d)))))
  (label 1748)
  (parent 784)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1749)
  (parent 784)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1750)
  (parent 784)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1751)
  (parent 784)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d)))))
  (label 1752)
  (parent 787)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1753)
  (parent 787)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1754)
  (parent 787)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1755)
  (parent 787)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1756)
  (parent 787)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nb-0 nd nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d)))))
  (label 1757)
  (parent 796)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1758)
  (parent 796)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1759)
  (parent 796)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1760)
  (parent 796)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1761)
  (parent 796)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation generalization deleted (2 0))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb-0 nc (pubk d)))
      (send (enc b c d na nb-0 nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc (pubk d)))))
  (label 1762)
  (parent 800)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1763)
  (parent 817)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1764)
  (parent 817)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nd nb-0 nc nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc-0) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc-0 (pubk d))
    (enc a b c na nb-0 nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc-0 nd (pubk a)))
      (send (enc nb nc-0 nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a b c na nb nc-0 (pubk d)))
      (send (enc b c d na nb nc-0 nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc (pubk d))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 1765)
  (parent 824)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-0 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1766)
  (parent 824)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-0 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1767)
  (parent 824)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-0 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1768)
  (parent 824)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-0 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-1 (pubk a)))))
  (label 1769)
  (parent 824)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b c na nb-1 nc-0 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1770)
  (parent 824)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nd nb nc nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 1771)
  (parent 825)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1772)
  (parent 825)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1773)
  (parent 825)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1774)
  (parent 825)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1775)
  (parent 825)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1776)
  (parent 833)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1777)
  (parent 833)
  (unrealized (0 1) (3 0) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (6 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1778)
  (parent 833)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1779)
  (parent 833)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1780)
  (parent 833)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1781)
  (parent 833)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0) (nd-1 nd-0))
    nc-0 (3 0) (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d)))))
  (label 1782)
  (parent 835)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp1 2) nc-1 (3 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-2 (pubk c)))))
  (label 1783)
  (parent 835)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (3 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1784)
  (parent 835)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (6 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 2 resp2 2) nc-1 (3 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1785)
  (parent 835)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (3 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1786)
  (parent 835)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nd-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (3 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-2 (pubk a)))))
  (label 1787)
  (parent 835)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1788)
  (parent 837)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (6 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1789)
  (parent 837)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1790)
  (parent 837)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 nd-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-2 (pubk a)))))
  (label 1791)
  (parent 837)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1792)
  (parent 839)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (6 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1793)
  (parent 839)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1794)
  (parent 839)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1795)
  (parent 839)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 5 resp2 2) nc-1 (6 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1796)
  (parent 841)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (6 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1797)
  (parent 843)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1798)
  (parent 843)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1799)
  (parent 843)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (6 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1800)
  (parent 843)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1801)
  (parent 843)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nd-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (6 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-1 nd-1 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-2 (pubk a)))))
  (label 1802)
  (parent 843)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nd nc nb text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation generalization deleted (2 0))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c)))))
  (label 1803)
  (parent 845)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation generalization deleted (4 0))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1804)
  (parent 849)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1805)
  (parent 857)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1806)
  (parent 857)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1807)
  (parent 857)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1808)
  (parent 858)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1809)
  (parent 858)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1810)
  (parent 858)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1811)
  (parent 860)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1812)
  (parent 860)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1813)
  (parent 860)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1814)
  (parent 861)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1815)
  (parent 861)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1816)
  (parent 861)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1817)
  (parent 862)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1818)
  (parent 862)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1819)
  (parent 862)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1820)
  (parent 863)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1821)
  (parent 863)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1822)
  (parent 863)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1823)
  (parent 864)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1824)
  (parent 864)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1825)
  (parent 864)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1826)
  (parent 864)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1827)
  (parent 864)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1828)
  (parent 864)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1829)
  (parent 865)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1830)
  (parent 865)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1831)
  (parent 865)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1832)
  (parent 865)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1833)
  (parent 865)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1834)
  (parent 865)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1835)
  (parent 866)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 1836)
  (parent 866)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1837)
  (parent 866)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1838)
  (parent 866)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1839)
  (parent 866)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1840)
  (parent 866)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1841)
  (parent 866)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1842)
  (parent 868)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1843)
  (parent 868)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1844)
  (parent 868)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1845)
  (parent 868)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1846)
  (parent 868)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1847)
  (parent 868)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 1848)
  (parent 870)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1849)
  (parent 870)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1850)
  (parent 870)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1851)
  (parent 870)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 1852)
  (parent 871)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1853)
  (parent 871)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1854)
  (parent 871)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1855)
  (parent 871)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1856)
  (parent 871)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1857)
  (parent 872)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1858)
  (parent 872)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1859)
  (parent 872)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1860)
  (parent 873)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1861)
  (parent 873)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1862)
  (parent 873)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (5 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1863)
  (parent 874)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1864)
  (parent 874)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1865)
  (parent 874)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-1 (pubk a)))))
  (label 1866)
  (parent 874)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1867)
  (parent 875)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1868)
  (parent 875)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1869)
  (parent 875)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1870)
  (parent 875)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1871)
  (parent 875)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-2 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-1 (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-1 (pubk d)))))
  (label 1872)
  (parent 877)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1873)
  (parent 877)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1874)
  (parent 877)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1875)
  (parent 877)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1876)
  (parent 877)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-1 (pubk a)))))
  (label 1877)
  (parent 877)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1878)
  (parent 877)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1879)
  (parent 878)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1880)
  (parent 878)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1881)
  (parent 878)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1882)
  (parent 880)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1883)
  (parent 880)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1884)
  (parent 880)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1885)
  (parent 880)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1886)
  (parent 880)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1887)
  (parent 881)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (6 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1888)
  (parent 881)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1889)
  (parent 881)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1890)
  (parent 883)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1891)
  (parent 883)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1892)
  (parent 883)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1893)
  (parent 883)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1894)
  (parent 884)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1895)
  (parent 884)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1896)
  (parent 899)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1897)
  (parent 899)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1898)
  (parent 899)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1899)
  (parent 901)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1900)
  (parent 901)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1901)
  (parent 901)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1902)
  (parent 901)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1903)
  (parent 901)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1904)
  (parent 902)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1905)
  (parent 902)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1906)
  (parent 904)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1907)
  (parent 904)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1908)
  (parent 904)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1909)
  (parent 904)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1910)
  (parent 905)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1911)
  (parent 905)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1912)
  (parent 909)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1913)
  (parent 909)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1914)
  (parent 909)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1915)
  (parent 910)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1916)
  (parent 910)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1917)
  (parent 910)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1918)
  (parent 912)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1919)
  (parent 912)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1920)
  (parent 913)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1921)
  (parent 913)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1922)
  (parent 913)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1923)
  (parent 914)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1924)
  (parent 914)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1925)
  (parent 914)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1926)
  (parent 915)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1927)
  (parent 915)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1928)
  (parent 915)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nc nd nb nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1929)
  (parent 916)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1930)
  (parent 916)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1931)
  (parent 916)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1932)
  (parent 916)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1933)
  (parent 916)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1934)
  (parent 916)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nc nd nb nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb-0 nc (pubk d))
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb (pubk c))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb-0 nc nd (pubk a)))
      (send (enc nb-0 nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc (pubk d))))
    ((recv (enc a b c na nb-0 nc (pubk d)))
      (send (enc b c d na nb-0 nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1935)
  (parent 917)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1936)
  (parent 917)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1937)
  (parent 917)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1938)
  (parent 917)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 1939)
  (parent 917)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-1 (pubk a)))))
  (label 1940)
  (parent 917)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 1941)
  (parent 918)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 1942)
  (parent 918)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1943)
  (parent 918)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1944)
  (parent 918)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1945)
  (parent 918)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 1946)
  (parent 918)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1947)
  (parent 918)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1948)
  (parent 921)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1949)
  (parent 921)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1950)
  (parent 921)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1951)
  (parent 921)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1952)
  (parent 921)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1953)
  (parent 922)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1954)
  (parent 922)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1955)
  (parent 922)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-2 (pubk d)))
      (send (enc b c d na nb-1 nc-2 nd-1 (pubk a)))))
  (label 1956)
  (parent 922)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 1957)
  (parent 937)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1958)
  (parent 937)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1959)
  (parent 937)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1960)
  (parent 937)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation generalization deleted (4 0))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd-0 (pubk a)))))
  (label 1961)
  (parent 940)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1962)
  (parent 942)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1963)
  (parent 942)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1964)
  (parent 943)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1965)
  (parent 943)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nd-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-2 (pubk a)))))
  (label 1966)
  (parent 944)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nd-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-2 (pubk a)))))
  (label 1967)
  (parent 944)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nd-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-2 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1968)
  (parent 944)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nd-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-1 nc-0)) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 1969)
  (parent 945)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (5 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1970)
  (parent 945)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1971)
  (parent 945)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nd-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-2 (pubk a)))))
  (label 1972)
  (parent 945)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1973)
  (parent 946)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 1974)
  (parent 946)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1975)
  (parent 946)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1976)
  (parent 946)
  (unrealized (0 1) (5 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1977)
  (parent 950)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1978)
  (parent 950)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 1979)
  (parent 950)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1980)
  (parent 950)
  (unrealized (0 1) (5 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nd-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 1981)
  (parent 951)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1982)
  (parent 951)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1983)
  (parent 951)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nd-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-2 (pubk a)))))
  (label 1984)
  (parent 951)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation generalization deleted (4 0))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd-0 (pubk a)))))
  (label 1985)
  (parent 953)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation generalization deleted (3 0))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb-0 (pubk c))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a)))))
  (label 1986)
  (parent 954)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-2 nb) (nc-0 nc) (nd-1 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b d nc nb (pubk c))
    (enc a b d nc nb-0 (pubk c)) (enc a b d nc nb-1 (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nb nc nd (pubk a))
    (enc b c d nc nb nc nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb-0 (pubk c))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd-0 (pubk a))))
    ((recv (enc a c d nc (pubk b)))
      (send (enc a b d nc nb-1 (pubk c)))))
  (label 1987)
  (parent 955)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-2 nb) (nc-0 nc) (nd-1 nd-0)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b d nc nb (pubk c))
    (enc a b d nc nb-0 (pubk c)) (enc a b d nc nb-1 (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nb nc nd (pubk a))
    (enc b c d nc nb nc nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd-0 (pubk a)))
      (send (enc nb nc nd-0 (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb-0 (pubk c))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd-0 (pubk a))))
    ((recv (enc a c d nc (pubk b)))
      (send (enc a b d nc nb-1 (pubk c)))))
  (label 1988)
  (parent 955)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nd-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a b d nc-0 nb-2 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))
    (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nd-1 (pubk c)))))
  (label 1989)
  (parent 955)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a b d nc-0 nb-2 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))
    (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nd-0 (pubk c)))))
  (label 1990)
  (parent 955)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a b d nc-0 nb-2 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))
    (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c))))
    ((recv (enc a b d nc-0 nb-2 (pubk c)))
      (send (enc a b c nc-0 nb-2 nc-1 (pubk d)))))
  (label 1991)
  (parent 955)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a b d nc-0 nb-2 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))
    (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 1992)
  (parent 955)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a b d nc-0 nb-2 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))
    (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 1993)
  (parent 955)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 nd-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-2) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a b d nc-0 nb-2 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))
    (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-2 (pubk a)))))
  (label 1994)
  (parent 955)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 1995)
  (parent 956)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 4 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 1996)
  (parent 956)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 1997)
  (parent 956)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 1998)
  (parent 957)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 4 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 1999)
  (parent 957)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 2000)
  (parent 957)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 49)
  (parent 23)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 74)
  (parent 34)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 80)
  (parent 36)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 87)
  (parent 38)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 101)
  (parent 44)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 105)
  (parent 45)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 109)
  (parent 46)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 112)
  (parent 47)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 114)
  (parent 47)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 121)
  (parent 51)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 137)
  (parent 57)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 162)
  (parent 67)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 174)
  (parent 70)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 176)
  (parent 70)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 182)
  (parent 72)
  (unrealized (0 1) (3 0) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nb-1 (pubk d)))))
  (label 184)
  (parent 73)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 185)
  (parent 73)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 188)
  (parent 75)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 191)
  (parent 76)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 201)
  (parent 78)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a b c na nb-2 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 203)
  (parent 79)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-2 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 204)
  (parent 79)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-2 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 205)
  (parent 79)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-2 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b c na nb-2 nc-1 (pubk d)))
      (send (enc b c d na nb-2 nc-1 nd-1 (pubk a)))))
  (label 206)
  (parent 79)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 209)
  (parent 82)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 213)
  (parent 83)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 216)
  (parent 84)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 219)
  (parent 85)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 221)
  (parent 86)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 222)
  (parent 86)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 223)
  (parent 86)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 243)
  (parent 92)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 246)
  (parent 94)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 252)
  (parent 97)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 253)
  (parent 98)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 255)
  (parent 98)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 256)
  (parent 98)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 260)
  (parent 99)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 261)
  (parent 99)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 263)
  (parent 100)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 264)
  (parent 100)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 266)
  (parent 100)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 267)
  (parent 100)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 271)
  (parent 102)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 273)
  (parent 102)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 275)
  (parent 103)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 276)
  (parent 103)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 278)
  (parent 104)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 280)
  (parent 104)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 281)
  (parent 104)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-2 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-2 nc-1 nd-1 (pubk a)))))
  (label 282)
  (parent 104)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 286)
  (parent 106)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 288)
  (parent 106)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nc-1 (pubk c)))))
  (label 290)
  (parent 107)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 291)
  (parent 107)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 293)
  (parent 107)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 294)
  (parent 107)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 297)
  (parent 108)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 300)
  (parent 110)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 301)
  (parent 110)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a b c na nb-2 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 303)
  (parent 111)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-2 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 304)
  (parent 111)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-2 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 306)
  (parent 111)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-2 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b c na nb-2 nc-1 (pubk d)))
      (send (enc b c d na nb-2 nc-1 nd-1 (pubk a)))))
  (label 307)
  (parent 111)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 309)
  (parent 113)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 310)
  (parent 113)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (6 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 311)
  (parent 113)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 316)
  (parent 116)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 317)
  (parent 116)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 318)
  (parent 116)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 319)
  (parent 116)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 322)
  (parent 118)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 325)
  (parent 119)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nb-1 (pubk d)))))
  (label 327)
  (parent 120)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nb-0 (pubk d)))))
  (label 328)
  (parent 120)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 330)
  (parent 120)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 331)
  (parent 120)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 344)
  (parent 124)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 356)
  (parent 127)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 364)
  (parent 130)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 370)
  (parent 132)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 372)
  (parent 134)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 374)
  (parent 135)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 375)
  (parent 135)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 376)
  (parent 135)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 378)
  (parent 136)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 379)
  (parent 136)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 381)
  (parent 136)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 382)
  (parent 136)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 398)
  (parent 144)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 405)
  (parent 151)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 411)
  (parent 154)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 418)
  (parent 156)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 426)
  (parent 158)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 429)
  (parent 159)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 432)
  (parent 160)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 434)
  (parent 160)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 436)
  (parent 161)
  (unrealized (0 1) (3 0) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 437)
  (parent 161)
  (unrealized (0 1) (3 0) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-2 (pubk d)))
      (send (enc b c d na nb-1 nc-2 nd-1 (pubk a)))))
  (label 438)
  (parent 161)
  (unrealized (0 1) (3 0) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 444)
  (parent 165)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 449)
  (parent 167)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 450)
  (parent 167)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 452)
  (parent 167)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 453)
  (parent 167)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 456)
  (parent 168)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 464)
  (parent 170)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 467)
  (parent 171)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 469)
  (parent 172)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 471)
  (parent 172)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 472)
  (parent 172)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 474)
  (parent 173)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 476)
  (parent 173)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 477)
  (parent 173)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 480)
  (parent 175)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 481)
  (parent 175)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 482)
  (parent 175)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 490)
  (parent 179)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 492)
  (parent 180)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 494)
  (parent 180)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-1 (pubk a)))))
  (label 495)
  (parent 180)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 499)
  (parent 181)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 501)
  (parent 183)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 502)
  (parent 183)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 503)
  (parent 183)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 505)
  (parent 186)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 506)
  (parent 186)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 508)
  (parent 187)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 509)
  (parent 187)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 510)
  (parent 187)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 511)
  (parent 187)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-2 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-2 nc-1 nd-1 (pubk a)))))
  (label 512)
  (parent 187)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nc-1 (pubk c)))))
  (label 514)
  (parent 189)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 515)
  (parent 189)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 516)
  (parent 189)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 518)
  (parent 189)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 519)
  (parent 189)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 522)
  (parent 190)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nb-3 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-3) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a b d nb-1 nb-3 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c))))
    ((recv (enc a b d nb-1 nb-3 (pubk c)))
      (send (enc a b c nb-1 nb-3 nc-1 (pubk d)))))
  (label 524)
  (parent 193)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nb-3 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a b d nb-1 nb-3 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 525)
  (parent 193)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nb-3 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a b d nb-1 nb-3 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 526)
  (parent 193)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nb-3 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a b d nb-1 nb-3 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 527)
  (parent 193)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nb-3 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((0 0) (7 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a b d nb-1 nb-3 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 528)
  (parent 193)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 532)
  (parent 194)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 536)
  (parent 195)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 540)
  (parent 196)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 544)
  (parent 197)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 546)
  (parent 198)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 547)
  (parent 198)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 548)
  (parent 198)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 549)
  (parent 198)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 552)
  (parent 199)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 553)
  (parent 199)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 556)
  (parent 200)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 558)
  (parent 202)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 559)
  (parent 202)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-1 (pubk a)))))
  (label 560)
  (parent 202)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 563)
  (parent 207)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 565)
  (parent 207)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 567)
  (parent 208)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 568)
  (parent 208)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 569)
  (parent 208)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b c na nb-2 nc-1 (pubk d)))
      (send (enc b c d na nb-2 nc-1 nd-1 (pubk a)))))
  (label 570)
  (parent 208)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 571)
  (parent 208)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 575)
  (parent 210)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 577)
  (parent 210)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 579)
  (parent 211)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 580)
  (parent 211)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 581)
  (parent 211)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 582)
  (parent 211)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 583)
  (parent 211)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 584)
  (parent 211)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 587)
  (parent 212)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 588)
  (parent 212)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 590)
  (parent 214)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 591)
  (parent 214)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 592)
  (parent 214)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 593)
  (parent 214)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 594)
  (parent 214)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 597)
  (parent 215)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 598)
  (parent 215)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 600)
  (parent 217)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 602)
  (parent 217)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 603)
  (parent 218)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nd-1 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 604)
  (parent 218)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nb-2 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 606)
  (parent 220)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nb-2 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 607)
  (parent 220)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nb-2 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 608)
  (parent 220)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nb-2 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 609)
  (parent 220)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 622)
  (parent 226)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 624)
  (parent 226)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 632)
  (parent 228)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 634)
  (parent 230)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 635)
  (parent 230)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 636)
  (parent 230)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 637)
  (parent 230)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 638)
  (parent 230)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 641)
  (parent 232)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 644)
  (parent 233)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 647)
  (parent 234)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 649)
  (parent 236)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 651)
  (parent 238)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 653)
  (parent 239)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 655)
  (parent 240)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 656)
  (parent 240)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 657)
  (parent 240)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-1 (pubk a)))))
  (label 658)
  (parent 240)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 662)
  (parent 241)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 663)
  (parent 241)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 665)
  (parent 242)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 666)
  (parent 242)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 667)
  (parent 242)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 668)
  (parent 242)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 669)
  (parent 242)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d)))))
  (label 671)
  (parent 244)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 672)
  (parent 244)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 673)
  (parent 245)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 674)
  (parent 245)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 677)
  (parent 248)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 678)
  (parent 251)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 681)
  (parent 251)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 687)
  (parent 257)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 689)
  (parent 257)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 690)
  (parent 257)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 691)
  (parent 257)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 693)
  (parent 258)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 695)
  (parent 258)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 696)
  (parent 258)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 697)
  (parent 258)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 699)
  (parent 259)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 700)
  (parent 259)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 701)
  (parent 259)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 703)
  (parent 259)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 704)
  (parent 259)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 706)
  (parent 262)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 707)
  (parent 262)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 708)
  (parent 262)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nb-2 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 709)
  (parent 262)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 711)
  (parent 265)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 712)
  (parent 265)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 713)
  (parent 265)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 714)
  (parent 265)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 715)
  (parent 265)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 717)
  (parent 268)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 718)
  (parent 268)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 719)
  (parent 268)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 722)
  (parent 269)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 723)
  (parent 269)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 725)
  (parent 269)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 727)
  (parent 270)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 728)
  (parent 270)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 729)
  (parent 270)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 731)
  (parent 270)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-2 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-2 nc-1 nd-1 (pubk a)))))
  (label 732)
  (parent 270)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 734)
  (parent 272)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 735)
  (parent 272)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 736)
  (parent 272)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 738)
  (parent 274)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 739)
  (parent 274)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 740)
  (parent 274)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-2 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-2 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-2 nc-1 nd-1 (pubk a)))))
  (label 741)
  (parent 274)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 743)
  (parent 277)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 744)
  (parent 277)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 745)
  (parent 277)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-1 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-0 nd-1 (pubk a)))))
  (label 746)
  (parent 277)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nc-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nc-1 (pubk c)))))
  (label 753)
  (parent 283)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 754)
  (parent 283)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 755)
  (parent 283)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 756)
  (parent 283)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nc-1 (pubk c)))))
  (label 758)
  (parent 284)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 759)
  (parent 284)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 760)
  (parent 284)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 762)
  (parent 284)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 763)
  (parent 284)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 767)
  (parent 285)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 769)
  (parent 285)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 771)
  (parent 287)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 772)
  (parent 287)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (6 0)
    (enc a b c nb-1 nb-0 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 773)
  (parent 287)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 775)
  (parent 289)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 776)
  (parent 289)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 777)
  (parent 289)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 778)
  (parent 289)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 785)
  (parent 295)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 786)
  (parent 295)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 788)
  (parent 296)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 789)
  (parent 296)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 790)
  (parent 296)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 791)
  (parent 296)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 793)
  (parent 298)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 794)
  (parent 298)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nb-1 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 795)
  (parent 298)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 797)
  (parent 299)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 798)
  (parent 299)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 799)
  (parent 299)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 801)
  (parent 302)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 802)
  (parent 302)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-1 (pubk a)))))
  (label 803)
  (parent 302)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 805)
  (parent 305)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 806)
  (parent 305)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 807)
  (parent 305)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b c na nb-2 nc-1 (pubk d)))
      (send (enc b c d na nb-2 nc-1 nd-1 (pubk a)))))
  (label 808)
  (parent 305)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 809)
  (parent 305)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 811)
  (parent 308)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 812)
  (parent 308)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 813)
  (parent 308)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b c na nb-2 nc-1 (pubk d)))
      (send (enc b c d na nb-2 nc-1 nd-1 (pubk a)))))
  (label 814)
  (parent 308)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-2 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 815)
  (parent 308)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 818)
  (parent 314)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 820)
  (parent 315)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 821)
  (parent 315)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 822)
  (parent 315)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 823)
  (parent 315)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 826)
  (parent 320)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 827)
  (parent 320)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 828)
  (parent 320)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 829)
  (parent 320)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-1 (pubk a)))))
  (label 830)
  (parent 320)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 831)
  (parent 321)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 832)
  (parent 321)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 834)
  (parent 323)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 838)
  (parent 323)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 840)
  (parent 324)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 844)
  (parent 324)
  (unrealized (0 1) (3 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 846)
  (parent 326)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 847)
  (parent 326)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 848)
  (parent 326)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 850)
  (parent 329)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 851)
  (parent 329)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 852)
  (parent 329)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 853)
  (parent 329)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 854)
  (parent 329)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 867)
  (parent 334)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 869)
  (parent 334)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 876)
  (parent 336)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 879)
  (parent 339)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 882)
  (parent 340)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 885)
  (parent 341)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 887)
  (parent 342)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 888)
  (parent 342)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 889)
  (parent 342)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 890)
  (parent 342)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 891)
  (parent 342)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 893)
  (parent 343)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 894)
  (parent 343)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 895)
  (parent 343)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 896)
  (parent 343)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 897)
  (parent 343)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 898)
  (parent 343)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 900)
  (parent 346)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 903)
  (parent 348)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 906)
  (parent 349)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 919)
  (parent 352)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 920)
  (parent 352)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 923)
  (parent 353)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 925)
  (parent 354)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 926)
  (parent 354)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 927)
  (parent 354)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 928)
  (parent 354)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-1 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-0 (pubk d))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-1 nc-0 (pubk d)))
      (send (enc b c d na nb-1 nc-0 nd-1 (pubk a)))))
  (label 929)
  (parent 354)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 931)
  (parent 355)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 932)
  (parent 355)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 933)
  (parent 355)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 934)
  (parent 355)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 935)
  (parent 355)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 936)
  (parent 355)
  (unrealized (0 1) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 938)
  (parent 358)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 939)
  (parent 358)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 947)
  (parent 360)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-1 (pubk d)))))
  (label 948)
  (parent 361)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nb-2 nc-1 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 949)
  (parent 361)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nd-1 nc-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 952)
  (parent 362)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))
